@@ -21,7 +21,7 @@ class Converter::Html5Converter < Converter::Base
     #latexmath:  INLINE_MATH_DELIMITERS[:latexmath] + [false],
   }).default = ['', '']
 
-  DropAnchorRx = /<(?:a\b[^>]*|\/a)>/
+  DropAnchorRx = %r(<(?:a\b[^>]*|/a)>)
   StemBreakRx = / *\\\n(?:\\?\n)*|\n\n+/
   if RUBY_ENGINE == 'opal'
     # NOTE In JavaScript, ^ matches the start of the string when the m flag is not set
@@ -48,44 +48,45 @@ class Converter::Html5Converter < Converter::Base
   end
 
   def convert node, transform = node.node_name, opts = nil
-    if transform == 'inline_quoted'; return convert_inline_quoted node
-    elsif transform == 'paragraph'; return convert_paragraph node
-    elsif transform == 'inline_anchor'; return convert_inline_anchor node
-    elsif transform == 'section'; return convert_section node
-    elsif transform == 'listing'; return convert_listing node
-    elsif transform == 'literal'; return convert_literal node
-    elsif transform == 'ulist'; return convert_ulist node
-    elsif transform == 'olist'; return convert_olist node
-    elsif transform == 'dlist'; return convert_dlist node
-    elsif transform == 'admonition'; return convert_admonition node
-    elsif transform == 'colist'; return convert_colist node
-    elsif transform == 'embedded'; return convert_embedded node
-    elsif transform == 'example'; return convert_example node
-    elsif transform == 'floating_title'; return convert_floating_title node
-    elsif transform == 'image'; return convert_image node
-    elsif transform == 'inline_break'; return convert_inline_break node
-    elsif transform == 'inline_button'; return convert_inline_button node
-    elsif transform == 'inline_callout'; return convert_inline_callout node
-    elsif transform == 'inline_footnote'; return convert_inline_footnote node
-    elsif transform == 'inline_image'; return convert_inline_image node
-    elsif transform == 'inline_indexterm'; return convert_inline_indexterm node
-    elsif transform == 'inline_kbd'; return convert_inline_kbd node
-    elsif transform == 'inline_menu'; return convert_inline_menu node
-    elsif transform == 'open'; return convert_open node
-    elsif transform == 'page_break'; return convert_page_break node
-    elsif transform == 'preamble'; return convert_preamble node
-    elsif transform == 'quote'; return convert_quote node
-    elsif transform == 'sidebar'; return convert_sidebar node
-    elsif transform == 'stem'; return convert_stem node
-    elsif transform == 'table'; return convert_table node
-    elsif transform == 'thematic_break'; return convert_thematic_break node
-    elsif transform == 'verse'; return convert_verse node
-    elsif transform == 'video'; return convert_video node
-    elsif transform == 'document'; return convert_document node
-    elsif transform == 'toc'; return convert_toc node
-    elsif transform == 'pass'; return convert_pass node
-    elsif transform == 'audio'; return convert_audio node
-    else; return super
+    case transform
+    when 'inline_quoted' then convert_inline_quoted node
+    when 'paragraph' then convert_paragraph node
+    when 'inline_anchor' then convert_inline_anchor node
+    when 'section' then convert_section node
+    when 'listing' then convert_listing node
+    when 'literal' then convert_literal node
+    when 'ulist' then convert_ulist node
+    when 'olist' then convert_olist node
+    when 'dlist' then convert_dlist node
+    when 'admonition' then convert_admonition node
+    when 'colist' then convert_colist node
+    when 'embedded' then convert_embedded node
+    when 'example' then convert_example node
+    when 'floating_title' then convert_floating_title node
+    when 'image' then convert_image node
+    when 'inline_break' then convert_inline_break node
+    when 'inline_button' then convert_inline_button node
+    when 'inline_callout' then convert_inline_callout node
+    when 'inline_footnote' then convert_inline_footnote node
+    when 'inline_image' then convert_inline_image node
+    when 'inline_indexterm' then convert_inline_indexterm node
+    when 'inline_kbd' then convert_inline_kbd node
+    when 'inline_menu' then convert_inline_menu node
+    when 'open' then convert_open node
+    when 'page_break' then convert_page_break node
+    when 'preamble' then convert_preamble node
+    when 'quote' then convert_quote node
+    when 'sidebar' then convert_sidebar node
+    when 'stem' then convert_stem node
+    when 'table' then convert_table node
+    when 'thematic_break' then convert_thematic_break node
+    when 'verse' then convert_verse node
+    when 'video' then convert_video node
+    when 'document' then convert_document node
+    when 'toc' then convert_toc node
+    when 'pass' then convert_pass node
+    when 'audio' then convert_audio node
+    else; super
     end
   end
 
@@ -350,9 +351,10 @@ MathJax.Hub.Register.StartupHook("AsciiMath Jax Ready", function () {
         stitle = section.captioned_title
       elsif section.numbered && slevel <= sectnumlevels
         if slevel < 2 && node.document.doctype == 'book'
-          if section.sectname == 'chapter'
+          case section.sectname
+          when 'chapter'
             stitle =  %(#{(signifier = node.document.attributes['chapter-signifier']) ? "#{signifier} " : ''}#{section.sectnum} #{section.title})
-          elsif section.sectname == 'part'
+          when 'part'
             stitle =  %(#{(signifier = node.document.attributes['part-signifier']) ? "#{signifier} " : ''}#{section.sectnum nil, ':'} #{section.title})
           else
             stitle = %(#{section.sectnum} #{section.title})
@@ -383,9 +385,10 @@ MathJax.Hub.Register.StartupHook("AsciiMath Jax Ready", function () {
       title = node.captioned_title
     elsif node.numbered && level <= (doc_attrs['sectnumlevels'] || 3).to_i
       if level < 2 && node.document.doctype == 'book'
-        if node.sectname == 'chapter'
+        case node.sectname
+        when 'chapter'
           title = %(#{(signifier = doc_attrs['chapter-signifier']) ? "#{signifier} " : ''}#{node.sectnum} #{node.title})
-        elsif node.sectname == 'part'
+        when 'part'
           title = %(#{(signifier = doc_attrs['part-signifier']) ? "#{signifier} " : ''}#{node.sectnum nil, ':'} #{node.title})
         else
           title = %(#{node.sectnum} #{node.title})
@@ -513,16 +516,16 @@ Your browser does not support the audio tag.
     result = []
     id_attribute = node.id ? %( id="#{node.id}") : ''
 
-    classes = case node.style
+    case node.style
     when 'qanda'
-      ['qlist', 'qanda', node.role]
+      classes = ['qlist', 'qanda', node.role]
     when 'horizontal'
-      ['hdlist', node.role]
+      classes = ['hdlist', node.role]
     else
-      ['dlist', node.style, node.role]
-    end.compact
+      classes = ['dlist', node.style, node.role]
+    end
 
-    class_attribute = %( class="#{classes.join ' '}")
+    class_attribute = %( class="#{classes.compact.join ' '}")
 
     result << %(<div#{id_attribute}#{class_attribute}>)
     result << %(<div class="title">#{node.title}</div>) if node.title?
@@ -578,12 +581,11 @@ Your browser does not support the audio tag.
         terms.each do |dt|
           result << %(<dt#{dt_style_attribute}>#{dt.text}</dt>)
         end
-        if dd
-          result << '<dd>'
-          result << %(<p>#{dd.text}</p>) if dd.text?
-          result << dd.content if dd.blocks?
-          result << '</dd>'
-        end
+        next unless dd
+        result << '<dd>'
+        result << %(<p>#{dd.text}</p>) if dd.text?
+        result << dd.content if dd.blocks?
+        result << '</dd>'
       end
       result << '</dl>'
     end
@@ -624,16 +626,18 @@ Your browser does not support the audio tag.
     target = node.attr 'target'
     width_attr = (node.attr? 'width') ? %( width="#{node.attr 'width'}") : ''
     height_attr = (node.attr? 'height') ? %( height="#{node.attr 'height'}") : ''
-    if ((node.attr? 'format', 'svg') || (target.include? '.svg')) && node.document.safe < SafeMode::SECURE &&
-        ((svg = (node.option? 'inline')) || (obj = (node.option? 'interactive')))
-      if svg
+    if ((node.attr? 'format', 'svg') || (target.include? '.svg')) && node.document.safe < SafeMode::SECURE
+      if node.option? 'inline'
         img = (read_svg_contents node, target) || %(<span class="alt">#{node.alt}</span>)
-      elsif obj
-        fallback = (node.attr? 'fallback') ? %(<img src="#{node.image_uri(node.attr 'fallback')}" alt="#{encode_attribute_value node.alt}"#{width_attr}#{height_attr}#{@void_element_slash}>) : %(<span class="alt">#{node.alt}</span>)
+      elsif node.option? 'interactive'
+        fallback = (node.attr? 'fallback') ? %(<img src="#{node.image_uri node.attr 'fallback'}" alt="#{encode_attribute_value node.alt}"#{width_attr}#{height_attr}#{@void_element_slash}>) : %(<span class="alt">#{node.alt}</span>)
         img = %(<object type="image/svg+xml" data="#{node.image_uri target}"#{width_attr}#{height_attr}>#{fallback}</object>)
+      else
+        img = %(<img src="#{node.image_uri target}" alt="#{encode_attribute_value node.alt}"#{width_attr}#{height_attr}#{@void_element_slash}>)
       end
+    else
+      img = %(<img src="#{node.image_uri target}" alt="#{encode_attribute_value node.alt}"#{width_attr}#{height_attr}#{@void_element_slash}>)
     end
-    img ||= %(<img src="#{node.image_uri target}" alt="#{encode_attribute_value node.alt}"#{width_attr}#{height_attr}#{@void_element_slash}>)
     img = %(<a class="image" href="#{node.attr 'link'}"#{(append_link_constraint_attrs node).join}>#{img}</a>) if node.attr? 'link'
     id_attr = node.id ? %( id="#{node.id}") : ''
     classes = ['imageblock']
@@ -671,7 +675,7 @@ Your browser does not support the audio tag.
     title_element = node.title? ? %(<div class="title">#{node.captioned_title}</div>\n) : ''
     %(<div#{id_attribute} class="listingblock#{(role = node.role) ? " #{role}" : ''}">
 #{title_element}<div class="content">
-#{syntax_hl ? (syntax_hl.format node, lang, opts) : pre_open + (node.content || '') + pre_close}
+#{syntax_hl ? (syntax_hl.format node, lang, opts) : pre_open + node.content + pre_close}
 </div>
 </div>)
   end
@@ -759,8 +763,8 @@ Your browser does not support the audio tag.
       logger.error 'partintro block can only be used when doctype is book and must be a child of a book part. Excluding block content.'
       ''
     else
-        id_attr = node.id ? %( id="#{node.id}") : ''
-        title_el = node.title? ? %(<div class="title">#{node.title}</div>\n) : ''
+      id_attr = node.id ? %( id="#{node.id}") : ''
+      title_el = node.title? ? %(<div class="title">#{node.title}</div>\n) : ''
       %(<div#{id_attr} class="openblock#{style && style != 'open' ? " #{style}" : ''}#{(role = node.role) ? " #{role}" : ''}">
 #{title_el}<div class="content">
 #{node.content}
@@ -857,20 +861,19 @@ Your browser does not support the audio tag.
     if (stripes = node.attr 'stripes', nil, 'table-stripes')
       classes << %(stripes-#{stripes})
     end
-    styles = []
+    style_attribute = ''
     if (autowidth = node.option? 'autowidth') && !(node.attr? 'width')
       classes << 'fit-content'
     elsif (tablewidth = node.attr 'tablepcwidth') == 100
       classes << 'stretch'
     else
-      styles << %(width: #{tablewidth}%;)
+      style_attribute = %( style="width: #{tablewidth}%;")
     end
     classes << (node.attr 'float') if node.attr? 'float'
     if (role = node.role)
       classes << role
     end
     class_attribute = %( class="#{classes.join ' '}")
-    style_attribute = styles.empty? ? '' : %( style="#{styles.join ' '}")
 
     result << %(<table#{id_attribute}#{class_attribute}#{style_attribute}>)
     result << %(<caption class="title">#{node.captioned_title}</caption>) if node.title?
@@ -1033,12 +1036,14 @@ Your browser does not support the audio tag.
       end
       start_anchor = (node.attr? 'start') ? %(#at=#{node.attr 'start'}) : ''
       delimiter = ['?']
+      target, hash = (node.attr 'target').split '/', 2
+      hash_param = (hash ||= node.attr 'hash') ? %(#{delimiter.pop || '&amp;'}h=#{hash}) : ''
       autoplay_param = (node.option? 'autoplay') ? %(#{delimiter.pop || '&amp;'}autoplay=1) : ''
       loop_param = (node.option? 'loop') ? %(#{delimiter.pop || '&amp;'}loop=1) : ''
       muted_param = (node.option? 'muted') ? %(#{delimiter.pop || '&amp;'}muted=1) : ''
       %(<div#{id_attribute}#{class_attribute}>#{title_element}
 <div class="content">
-<iframe#{width_attribute}#{height_attribute} src="#{asset_uri_scheme}//player.vimeo.com/video/#{node.attr 'target'}#{autoplay_param}#{loop_param}#{muted_param}#{start_anchor}" frameborder="0"#{(node.option? 'nofullscreen') ? '' : (append_boolean_attribute 'allowfullscreen', xml)}></iframe>
+<iframe#{width_attribute}#{height_attribute} src="#{asset_uri_scheme}//player.vimeo.com/video/#{target}#{hash_param}#{autoplay_param}#{loop_param}#{muted_param}#{start_anchor}" frameborder="0"#{(node.option? 'nofullscreen') ? '' : (append_boolean_attribute 'allowfullscreen', xml)}></iframe>
 </div>
 </div>)
     when 'youtube'
@@ -1074,7 +1079,7 @@ Your browser does not support the audio tag.
         target, playlist = target.split ',', 2
         if (playlist ||= (node.attr 'playlist'))
           # INFO playlist bar doesn't appear in Firefox unless showinfo=1 and modestbranding=1
-          list_param = %(&amp;playlist=#{playlist})
+          list_param = %(&amp;playlist=#{target},#{playlist})
         else
           # NOTE for loop to work, playlist must be specified; use VIDEO_ID if there's no explicit playlist
           list_param = has_loop_param ? %(&amp;playlist=#{target}) : ''
@@ -1178,40 +1183,49 @@ Your browser does not support the video tag.
   end
 
   def convert_inline_image node
-    if (type = node.type || 'image') == 'icon' && (node.document.attr? 'icons', 'font')
-      class_attr_val = %(fa fa-#{node.target})
-      { 'size' => 'fa-', 'rotate' => 'fa-rotate-', 'flip' => 'fa-flip-' }.each do |key, prefix|
-        class_attr_val = %(#{class_attr_val} #{prefix}#{node.attr key}) if node.attr? key
-      end
-      title_attr = (node.attr? 'title') ? %( title="#{node.attr 'title'}") : ''
-      img = %(<i class="#{class_attr_val}"#{title_attr}></i>)
-    elsif type == 'icon' && !(node.document.attr? 'icons')
-      img = %([#{node.alt}])
-    else
-      target = node.target
-      attrs = ['width', 'height', 'title'].map {|name| (node.attr? name) ? %( #{name}="#{node.attr name}") : '' }.join
-      if type != 'icon' && ((node.attr? 'format', 'svg') || (target.include? '.svg')) &&
-          node.document.safe < SafeMode::SECURE && ((svg = (node.option? 'inline')) || (obj = (node.option? 'interactive')))
-        if svg
-          img = (read_svg_contents node, target) || %(<span class="alt">#{node.alt}</span>)
-        elsif obj
-          fallback = (node.attr? 'fallback') ? %(<img src="#{node.image_uri(node.attr 'fallback')}" alt="#{encode_attribute_value node.alt}"#{attrs}#{@void_element_slash}>) : %(<span class="alt">#{node.alt}</span>)
-          img = %(<object type="image/svg+xml" data="#{node.image_uri target}"#{attrs}>#{fallback}</object>)
+    target = node.target
+    if (type = node.type || 'image') == 'icon'
+      if (icons = node.document.attr 'icons') == 'font'
+        i_class_attr_val = %(fa fa-#{target})
+        i_class_attr_val = %(#{i_class_attr_val} fa-#{node.attr 'size'}) if node.attr? 'size'
+        if node.attr? 'flip'
+          i_class_attr_val = %(#{i_class_attr_val} fa-flip-#{node.attr 'flip'})
+        elsif node.attr? 'rotate'
+          i_class_attr_val = %(#{i_class_attr_val} fa-rotate-#{node.attr 'rotate'})
         end
+        attrs = (node.attr? 'title') ? %( title="#{node.attr 'title'}") : ''
+        img = %(<i class="#{i_class_attr_val}"#{attrs}></i>)
+      elsif icons
+        attrs = (node.attr? 'width') ? %( width="#{node.attr 'width'}") : ''
+        attrs = %(#{attrs} height="#{node.attr 'height'}") if node.attr? 'height'
+        attrs = %(#{attrs} title="#{node.attr 'title'}") if node.attr? 'title'
+        img = %(<img src="#{node.icon_uri target}" alt="#{encode_attribute_value node.alt}"#{attrs}#{@void_element_slash}>)
+      else
+        img = %([#{node.alt}&#93;)
+      end
+    else
+      attrs = (node.attr? 'width') ? %( width="#{node.attr 'width'}") : ''
+      attrs = %(#{attrs} height="#{node.attr 'height'}") if node.attr? 'height'
+      attrs = %(#{attrs} title="#{node.attr 'title'}") if node.attr? 'title'
+      if ((node.attr? 'format', 'svg') || (target.include? '.svg')) && node.document.safe < SafeMode::SECURE
+        if node.option? 'inline'
+          img = (read_svg_contents node, target) || %(<span class="alt">#{node.alt}</span>)
+        elsif node.option? 'interactive'
+          fallback = (node.attr? 'fallback') ? %(<img src="#{node.image_uri node.attr 'fallback'}" alt="#{encode_attribute_value node.alt}"#{attrs}#{@void_element_slash}>) : %(<span class="alt">#{node.alt}</span>)
+          img = %(<object type="image/svg+xml" data="#{node.image_uri target}"#{attrs}>#{fallback}</object>)
+        else
+          img = %(<img src="#{node.image_uri target}" alt="#{encode_attribute_value node.alt}"#{attrs}#{@void_element_slash}>)
+        end
+      else
+        img = %(<img src="#{node.image_uri target}" alt="#{encode_attribute_value node.alt}"#{attrs}#{@void_element_slash}>)
       end
-      img ||= %(<img src="#{type == 'icon' ? (node.icon_uri target) : (node.image_uri target)}" alt="#{encode_attribute_value node.alt}"#{attrs}#{@void_element_slash}>)
     end
     img = %(<a class="image" href="#{node.attr 'link'}"#{(append_link_constraint_attrs node).join}>#{img}</a>) if node.attr? 'link'
+    class_attr_val = type
     if (role = node.role)
-      if node.attr? 'float'
-        class_attr_val = %(#{type} #{node.attr 'float'} #{role})
-      else
-        class_attr_val = %(#{type} #{role})
-      end
+      class_attr_val = (node.attr? 'float') ? %(#{class_attr_val} #{node.attr 'float'} #{role}) : %(#{class_attr_val} #{role})
     elsif node.attr? 'float'
-      class_attr_val = %(#{type} #{node.attr 'float'})
-    else
-      class_attr_val = type
+      class_attr_val = %(#{class_attr_val} #{node.attr 'float'})
     end
     %(<span class="#{class_attr_val}">#{img}</span>)
   end
@@ -1325,8 +1339,12 @@ Your browser does not support the video tag.
   end
 
   # NOTE adapt to older converters that relied on unprefixed method names
-  def method_missing id, *params
-    !((name = id.to_s).start_with? 'convert_') && (handles? name) ? (send %(convert_#{name}), *params) : super
+  def method_missing id, *args
+    !((name = id.to_s).start_with? 'convert_') && (handles? name) ? (send %(convert_#{name}), *args) : super
+  end
+
+  def respond_to_missing? id, *options
+    !((name = id.to_s).start_with? 'convert_') && (handles? name)
   end
 end
 end
